      SUBROUTINE LIOP (EINST,WEIGHTI,WEIGHTJ,I,J,
     $           ND,XLAM,ENTOT,POPNUM,RSTAR,OPAL,ETAL,VDOP)
C***********************************************************************
C***  CALCULATES THE LINE OPACITY OPAL AND THE EMISSIVITY ETAL FOR ONE LINE
C***  AT ALL DEPTH POINTS
C***  PHYSICAL DIMENSION OF OPAL : PER RSTAR AND PER DELTA-NUE-DOPPLER
C***    ( THE LATTER WILL BE CANCELLED OUT BY THE NORMALISED PROFILE FUNCTION )
C***  GIVEN QUANTITIES : EINST = EINSTEIN COEFFICIENT A(UP-LOW)  (PER SECOND)
C***                     WEIGHTI, WEIGHTJ = STATISTICAL WEIGHTS (UP, LOW)
C***                     POPNUM(L,J) = RELATIVE POPULATION NUMBERS
C***                     ENTOT(L) = TOTAL NUMBER DENSITY
C***********************************************************************
 
      DIMENSION ENTOT(ND),OPAL(ND),ETAL(ND),POPNUM(ND,1)

C***  C2 = 2 * H * C     ( CGS UNITS )
      DATA C2 / 3.9724E-16 /
C***  PI8 = 8*PI
      DATA PI8 /25.1327412288 /

C***  FOR EXACT COMPATIBILITY, CALCULATE C3 = H*C/(4*PI) :
      C3=C2/PI8

C***  WAVELENGTH IN CENTIMETER
      XLAMCM=XLAM/1.E8

C***  DND= DELTA-NUE-DOPPLER  (HERTZ)
      DND=VDOP/XLAM*1.E13

      EMINDU=EINST*XLAMCM*XLAMCM/PI8*RSTAR
      ABSORP = EMINDU*WEIGHTJ/WEIGHTI
      EMSPON=C3*RSTAR*EINST/XLAMCM

      DO L=1,ND
         ENI=POPNUM(L,I)*ENTOT(L)
         ENJ=POPNUM(L,J)*ENTOT(L)
C***     Set emissivities zero if both levels are equal (=POPMIN)
         IF (ENI .EQ. ENJ) ENJ=.0
         OPAL(L)=(ENI*ABSORP-ENJ*EMINDU)/DND
         ETAL(L)=ENJ*EMSPON/DND
      ENDDO

      RETURN
      END
